@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 from __future__ import division, print_function, absolute_import
 
 from os.path import join
@@ -9,8 +8,6 @@ def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
     from scipy._build_utils import get_g77_abi_wrappers, get_sgemv_fix
 
-    config = Configuration('arpack',parent_package,top_path)
-
     lapack_opt = get_info('lapack_opt')
 
     if not lapack_opt:
@@ -20,7 +17,6 @@ def configuration(parent_package='',top_path=None):
 
     arpack_sources = [join('ARPACK','SRC', '*.f')]
     arpack_sources.extend([join('ARPACK','UTIL', '*.f')])
-    arpack_sources.extend([join('ARPACK','LAPACK', '*.f')])
 
     arpack_sources += get_g77_abi_wrappers(lapack_opt)
 
@@ -37,6 +33,10 @@ def configuration(parent_package='',top_path=None):
                          )
 
     config.add_data_dir('tests')
+
+    # Add license files
+    config.add_data_files('ARPACK/COPYING')
+
     return config
 
 if __name__ == '__main__':
